// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "SceneItemComponent.hpp"

namespace Dawn {
  typedef int32_t sceneitemid_t;

  class Scene;

  class SceneItem :
    public std::enable_shared_from_this<SceneItem>
  {
    private:
      std::vector<std::shared_ptr<SceneItemComponent>> components;

    public:
      std::weak_ptr<Scene> scene;
      sceneitemid_t id;

      SceneItem(std::weak_ptr<Scene> scene, sceneitemid_t id);

      void init();

      template<class T>
      std::shared_ptr<T> addComponent() {
        auto component = std::make_shared<T>(weak_from_this());
        this->components.push_back(component);
        return component;
      }

      template<class T>
      std::shared_ptr<T> getComponent() {
        auto it = this->components.begin();
        while(it != this->components.end()) {
          auto castedAs = std::dynamic_pointer_cast<T>(*it);
          if(castedAs != nullptr) return castedAs;
          ++it;
        }
        return nullptr;
      }
      
      ~SceneItem();
  };
}