// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "Camera.hpp"

using namespace Dawn;

Camera::Camera(std::weak_ptr<SceneItem> item) : SceneItemComponent(item) {
}

void Camera::updateProjection() {
  switch(this->type) {
    case CAMERA_TYPE_ORTHONOGRAPHIC:
      this->projection = glm::ortho(
        this->orthoLeft,
        this->orthoRight,
        this->orthoBottom,
        this->orthoTop,
        this->clipNear,
        this->clipFar
      );
      break;

    case CAMERA_TYPE_PERSPECTIVE:
      this->projection = glm::perspective(
        this->fov,
        this->getAspect(),
        this->clipNear,
        this->clipFar
      );
      break;
  }
}

float_t Camera::getAspect() {
  return 16.0f / 9.0f;
}

void Camera::start() {
  this->updateProjection();
}