// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "DawnPokerGame.hpp"

using namespace Dawn;

DawnGame::DawnGame(std::weak_ptr<DawnHost> host) :
  renderManager(weak_from_this())
{
  this->host = host;
}

int32_t DawnGame::init() {
  this->renderManager.init();

  this->scene = std::make_shared<Scene>(weak_from_this());

  auto cameraObject = this->scene->createSceneItem();
  auto camera = cameraObject->addComponent<Camera>();

  auto cubeObject = this->scene->createSceneItem();
  auto cubeMeshRenderer = cubeObject->addComponent<MeshRenderer>();
  cubeMeshRenderer->mesh = std::make_shared<Mesh>();
  TriangleMesh::createTriangleMesh(cubeMeshRenderer->mesh);

  return DAWN_GAME_INIT_RESULT_SUCCESS;
}

int32_t DawnGame::update(float_t delta) {
  this->renderManager.update();
  if(this->scene != nullptr) this->scene->update();

  auto meshRenderer = this->scene->findComponent<MeshRenderer>();
  if(meshRenderer != nullptr) {
    std::cout << "REndering" << std::endl;
    meshRenderer->mesh->draw(MESH_DRAW_MODE_TRIANGLES, 0, -1);
  }

  return DAWN_GAME_UPDATE_RESULT_SUCCESS;
}

DawnGame::~DawnGame() {
  
}